package uniolunisaar.adam.tests.logics.mccformula;

import java.io.IOException;
import java.util.Map;
import javax.xml.parsers.ParserConfigurationException;
import org.testng.Assert;
import org.testng.annotations.Test;
import org.xml.sax.SAXException;
import uniol.apt.adt.pn.PetriNet;
import uniol.apt.io.parser.ParseException;
import uniol.apt.io.parser.impl.PnmlPNParser;
import uniol.apt.io.renderer.RenderException;
import uniol.apt.io.renderer.impl.LoLAPNRenderer;
import uniolunisaar.adam.ds.logics.ltl.ILTLFormula;
import uniolunisaar.adam.exceptions.logics.NotSubstitutableException;
import uniolunisaar.adam.logic.parser.logics.mccformula.MCCXMLFormulaParser;
import uniolunisaar.adam.tools.Tools;

/**
 *
 * @author Manuel Gieseking
 */
@Test
public class TestMCCLTL {

    private static final String inputDir = System.getProperty("examplesfolder") + "/modelchecking/ltl/mcc/";

    @Test
    public void firstTests() throws NotSubstitutableException, SAXException, ParserConfigurationException, IOException, ParseException, RenderException {
        String formula = "<?xml version=\"1.0\"?>\n"
                + "<property-set xmlns=\"http://mcc.lip6.fr/\">\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-00</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <until>\n"
                + "          <before>\n"
                + "            <integer-le>\n"
                + "              <tokens-count>\n"
                + "                <place>p232</place>\n"
                + "              </tokens-count>\n"
                + "              <tokens-count>\n"
                + "                <place>p396</place>\n"
                + "              </tokens-count>\n"
                + "            </integer-le>\n"
                + "          </before>\n"
                + "          <reach>\n"
                + "            <until>\n"
                + "              <before>\n"
                + "                <integer-le>\n"
                + "                  <integer-constant>2</integer-constant>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p45</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </before>\n"
                + "              <reach>\n"
                + "                <integer-le>\n"
                + "                  <integer-constant>1</integer-constant>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p23</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </reach>\n"
                + "            </until>\n"
                + "          </reach>\n"
                + "        </until>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-01</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <finally>\n"
                + "          <next>\n"
                + "            <globally>\n"
                + "              <globally>\n"
                + "                <integer-le>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p151</place>\n"
                + "                  </tokens-count>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p173</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </globally>\n"
                + "            </globally>\n"
                + "          </next>\n"
                + "        </finally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-02</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <next>\n"
                + "          <globally>\n"
                + "            <until>\n"
                + "              <before>\n"
                + "                <integer-le>\n"
                + "                  <integer-constant>1</integer-constant>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p107</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </before>\n"
                + "              <reach>\n"
                + "                <integer-le>\n"
                + "                  <integer-constant>3</integer-constant>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p44</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </reach>\n"
                + "            </until>\n"
                + "          </globally>\n"
                + "        </next>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-03</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <globally>\n"
                + "          <next>\n"
                + "            <integer-le>\n"
                + "              <integer-constant>1</integer-constant>\n"
                + "              <tokens-count>\n"
                + "                <place>p203</place>\n"
                + "              </tokens-count>\n"
                + "            </integer-le>\n"
                + "          </next>\n"
                + "        </globally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-04</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <finally>\n"
                + "          <next>\n"
                + "            <next>\n"
                + "              <finally>\n"
                + "                <integer-le>\n"
                + "                  <integer-constant>2</integer-constant>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p70</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </finally>\n"
                + "            </next>\n"
                + "          </next>\n"
                + "        </finally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-05</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <finally>\n"
                + "          <next>\n"
                + "            <finally>\n"
                + "              <next>\n"
                + "                <integer-le>\n"
                + "                  <integer-constant>3</integer-constant>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p224</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </next>\n"
                + "            </finally>\n"
                + "          </next>\n"
                + "        </finally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-06</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <globally>\n"
                + "          <until>\n"
                + "            <before>\n"
                + "              <integer-le>\n"
                + "                <tokens-count>\n"
                + "                  <place>p139</place>\n"
                + "                </tokens-count>\n"
                + "                <tokens-count>\n"
                + "                  <place>p328</place>\n"
                + "                </tokens-count>\n"
                + "              </integer-le>\n"
                + "            </before>\n"
                + "            <reach>\n"
                + "              <globally>\n"
                + "                <integer-le>\n"
                + "                  <integer-constant>2</integer-constant>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p359</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </globally>\n"
                + "            </reach>\n"
                + "          </until>\n"
                + "        </globally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-07</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <until>\n"
                + "          <before>\n"
                + "            <integer-le>\n"
                + "              <tokens-count>\n"
                + "                <place>p403</place>\n"
                + "              </tokens-count>\n"
                + "              <tokens-count>\n"
                + "                <place>p118</place>\n"
                + "              </tokens-count>\n"
                + "            </integer-le>\n"
                + "          </before>\n"
                + "          <reach>\n"
                + "            <finally>\n"
                + "              <integer-le>\n"
                + "                <integer-constant>2</integer-constant>\n"
                + "                <tokens-count>\n"
                + "                  <place>p336</place>\n"
                + "                </tokens-count>\n"
                + "              </integer-le>\n"
                + "            </finally>\n"
                + "          </reach>\n"
                + "        </until>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-08</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <finally>\n"
                + "          <globally>\n"
                + "            <next>\n"
                + "              <globally>\n"
                + "                <integer-le>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p372</place>\n"
                + "                  </tokens-count>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p103</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </globally>\n"
                + "            </next>\n"
                + "          </globally>\n"
                + "        </finally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-09</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <integer-le>\n"
                + "          <tokens-count>\n"
                + "            <place>p167</place>\n"
                + "          </tokens-count>\n"
                + "          <tokens-count>\n"
                + "            <place>p150</place>\n"
                + "          </tokens-count>\n"
                + "        </integer-le>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-10</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <until>\n"
                + "          <before>\n"
                + "            <finally>\n"
                + "              <globally>\n"
                + "                <integer-le>\n"
                + "                  <integer-constant>1</integer-constant>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p299</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </globally>\n"
                + "            </finally>\n"
                + "          </before>\n"
                + "          <reach>\n"
                + "            <integer-le>\n"
                + "              <tokens-count>\n"
                + "                <place>p351</place>\n"
                + "              </tokens-count>\n"
                + "              <tokens-count>\n"
                + "                <place>p418</place>\n"
                + "              </tokens-count>\n"
                + "            </integer-le>\n"
                + "          </reach>\n"
                + "        </until>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-11</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <until>\n"
                + "          <before>\n"
                + "            <until>\n"
                + "              <before>\n"
                + "                <integer-le>\n"
                + "                  <integer-constant>1</integer-constant>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p318</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </before>\n"
                + "              <reach>\n"
                + "                <integer-le>\n"
                + "                  <integer-constant>3</integer-constant>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p39</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </reach>\n"
                + "            </until>\n"
                + "          </before>\n"
                + "          <reach>\n"
                + "            <integer-le>\n"
                + "              <tokens-count>\n"
                + "                <place>p53</place>\n"
                + "              </tokens-count>\n"
                + "              <tokens-count>\n"
                + "                <place>p223</place>\n"
                + "              </tokens-count>\n"
                + "            </integer-le>\n"
                + "          </reach>\n"
                + "        </until>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-12</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <finally>\n"
                + "          <until>\n"
                + "            <before>\n"
                + "              <integer-le>\n"
                + "                <tokens-count>\n"
                + "                  <place>p17</place>\n"
                + "                </tokens-count>\n"
                + "                <tokens-count>\n"
                + "                  <place>p72</place>\n"
                + "                </tokens-count>\n"
                + "              </integer-le>\n"
                + "            </before>\n"
                + "            <reach>\n"
                + "              <integer-le>\n"
                + "                <tokens-count>\n"
                + "                  <place>p298</place>\n"
                + "                </tokens-count>\n"
                + "                <tokens-count>\n"
                + "                  <place>p330</place>\n"
                + "                </tokens-count>\n"
                + "              </integer-le>\n"
                + "            </reach>\n"
                + "          </until>\n"
                + "        </finally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-13</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <next>\n"
                + "          <finally>\n"
                + "            <finally>\n"
                + "              <next>\n"
                + "                <integer-le>\n"
                + "                  <integer-constant>1</integer-constant>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p416</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </next>\n"
                + "            </finally>\n"
                + "          </finally>\n"
                + "        </next>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-14</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <integer-le>\n"
                + "          <tokens-count>\n"
                + "            <place>p273</place>\n"
                + "          </tokens-count>\n"
                + "          <tokens-count>\n"
                + "            <place>p143</place>\n"
                + "          </tokens-count>\n"
                + "        </integer-le>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>ASLink-PT-01a-LTLCardinality-15</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <until>\n"
                + "          <before>\n"
                + "            <next>\n"
                + "              <finally>\n"
                + "                <integer-le>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p395</place>\n"
                + "                  </tokens-count>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p61</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </finally>\n"
                + "            </next>\n"
                + "          </before>\n"
                + "          <reach>\n"
                + "            <until>\n"
                + "              <before>\n"
                + "                <integer-le>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p325</place>\n"
                + "                  </tokens-count>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p311</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </before>\n"
                + "              <reach>\n"
                + "                <integer-le>\n"
                + "                  <integer-constant>3</integer-constant>\n"
                + "                  <tokens-count>\n"
                + "                    <place>p400</place>\n"
                + "                  </tokens-count>\n"
                + "                </integer-le>\n"
                + "              </reach>\n"
                + "            </until>\n"
                + "          </reach>\n"
                + "        </until>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "</property-set>";
        PetriNet net = new PnmlPNParser().parseFile(inputDir + "/ASLink/PT/aslink_01_a.pnml");
        try {
            MCCXMLFormulaParser.parseLTL(formula, net);
            Assert.fail("Shouldn't reach this state.");
        } catch (ParseException e) {
            Assert.assertEquals(e.getMessage(), "Currently only safe nets are supported.");
        }

        String safeFormula = formula.replaceAll("<integer-constant>\\d+</integer-constant>", "<integer-constant>1</integer-constant>");
        Map<String, ILTLFormula> f = MCCXMLFormulaParser.parseLTL(safeFormula, net);
        String[] check = new String[]{
            "(¬ (p232 ⋏ ¬ p396) 𝓤 (p45 𝓤 p23))",
            "◇ ◯ ⬜ ⬜ ¬ (p151 ⋏ ¬ p173)",
            "◯ ⬜ (p107 𝓤 p44)",
            "⬜ ◯ p203",
            "◇ ◯ ◯ ◇ p70",
            "◇ ◯ ◇ ◯ p224",
            "⬜ (¬ (p139 ⋏ ¬ p328) 𝓤 ⬜ p359)",
            "(¬ (p403 ⋏ ¬ p118) 𝓤 ◇ p336)",
            "◇ ⬜ ◯ ⬜ ¬ (p372 ⋏ ¬ p103)",
            "¬ (p167 ⋏ ¬ p150)",
            "(◇ ⬜ p299 𝓤 ¬ (p351 ⋏ ¬ p418))",
            "((p318 𝓤 p39) 𝓤 ¬ (p53 ⋏ ¬ p223))",
            "◇ (¬ (p17 ⋏ ¬ p72) 𝓤 ¬ (p298 ⋏ ¬ p330))",
            "◯ ◇ ◇ ◯ p416",
            "¬ (p273 ⋏ ¬ p143)",
            "(◯ ◇ ¬ (p395 ⋏ ¬ p61) 𝓤 (¬ (p325 ⋏ ¬ p311) 𝓤 p400))"
        };
        int i = 0;
        for (ILTLFormula iLTLFormula : f.values()) {
            Assert.assertEquals(iLTLFormula.toSymbolString(), check[i++]);
        }

        String out = new LoLAPNRenderer().render(net);
        Tools.saveFile("lolaOut", out);
    }

    @Test
    public void fireability() throws ParseException, IOException, SAXException, ParserConfigurationException {
        String formulas = "<?xml version=\"1.0\"?>\n"
                + "<property-set xmlns=\"http://mcc.lip6.fr/\">\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-0</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <is-fireable>\n"
                + "          <transition>End_Ext_Acc_4_1</transition>\n"
                + "          <transition>End_Ext_Acc_5_1</transition>\n"
                + "          <transition>End_Ext_Acc_2_1</transition>\n"
                + "          <transition>End_Ext_Acc_3_1</transition>\n"
                + "          <transition>End_Ext_Acc_5_2</transition>\n"
                + "          <transition>End_Ext_Acc_4_2</transition>\n"
                + "          <transition>End_Ext_Acc_3_2</transition>\n"
                + "          <transition>End_Ext_Acc_1_2</transition>\n"
                + "          <transition>End_Ext_Acc_5_3</transition>\n"
                + "          <transition>End_Ext_Acc_4_3</transition>\n"
                + "          <transition>End_Ext_Acc_2_3</transition>\n"
                + "          <transition>End_Ext_Acc_1_3</transition>\n"
                + "          <transition>End_Ext_Acc_5_4</transition>\n"
                + "          <transition>End_Ext_Acc_3_4</transition>\n"
                + "          <transition>End_Ext_Acc_2_4</transition>\n"
                + "          <transition>End_Ext_Acc_1_4</transition>\n"
                + "          <transition>End_Ext_Acc_4_5</transition>\n"
                + "          <transition>End_Ext_Acc_3_5</transition>\n"
                + "          <transition>End_Ext_Acc_2_5</transition>\n"
                + "          <transition>End_Ext_Acc_1_5</transition>\n"
                + "        </is-fireable>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-1</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <is-fireable>\n"
                + "          <transition>Req_Ext_Acc_2</transition>\n"
                + "          <transition>Req_Ext_Acc_1</transition>\n"
                + "          <transition>Req_Ext_Acc_4</transition>\n"
                + "          <transition>Req_Ext_Acc_3</transition>\n"
                + "          <transition>Req_Ext_Acc_5</transition>\n"
                + "        </is-fireable>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-2</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <is-fireable>\n"
                + "          <transition>Begin_Own_Acc_2</transition>\n"
                + "          <transition>Begin_Own_Acc_1</transition>\n"
                + "          <transition>Begin_Own_Acc_5</transition>\n"
                + "          <transition>Begin_Own_Acc_3</transition>\n"
                + "          <transition>Begin_Own_Acc_4</transition>\n"
                + "        </is-fireable>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-3</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <until>\n"
                + "          <before>\n"
                + "            <finally>\n"
                + "              <is-fireable>\n"
                + "                <transition>Req_Ext_Acc_2</transition>\n"
                + "                <transition>Req_Ext_Acc_1</transition>\n"
                + "                <transition>Req_Ext_Acc_4</transition>\n"
                + "                <transition>Req_Ext_Acc_3</transition>\n"
                + "                <transition>Req_Ext_Acc_5</transition>\n"
                + "              </is-fireable>\n"
                + "            </finally>\n"
                + "          </before>\n"
                + "          <reach>\n"
                + "            <is-fireable>\n"
                + "              <transition>Begin_Ext_Acc_1_2</transition>\n"
                + "              <transition>Begin_Ext_Acc_3_2</transition>\n"
                + "              <transition>Begin_Ext_Acc_4_2</transition>\n"
                + "              <transition>Begin_Ext_Acc_5_2</transition>\n"
                + "              <transition>Begin_Ext_Acc_2_1</transition>\n"
                + "              <transition>Begin_Ext_Acc_3_1</transition>\n"
                + "              <transition>Begin_Ext_Acc_4_1</transition>\n"
                + "              <transition>Begin_Ext_Acc_5_1</transition>\n"
                + "              <transition>Begin_Ext_Acc_1_4</transition>\n"
                + "              <transition>Begin_Ext_Acc_2_4</transition>\n"
                + "              <transition>Begin_Ext_Acc_3_4</transition>\n"
                + "              <transition>Begin_Ext_Acc_5_4</transition>\n"
                + "              <transition>Begin_Ext_Acc_1_3</transition>\n"
                + "              <transition>Begin_Ext_Acc_2_3</transition>\n"
                + "              <transition>Begin_Ext_Acc_4_3</transition>\n"
                + "              <transition>Begin_Ext_Acc_5_3</transition>\n"
                + "              <transition>Begin_Ext_Acc_2_5</transition>\n"
                + "              <transition>Begin_Ext_Acc_1_5</transition>\n"
                + "              <transition>Begin_Ext_Acc_4_5</transition>\n"
                + "              <transition>Begin_Ext_Acc_3_5</transition>\n"
                + "            </is-fireable>\n"
                + "          </reach>\n"
                + "        </until>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-4</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <is-fireable>\n"
                + "          <transition>End_Ext_Acc_4_1</transition>\n"
                + "          <transition>End_Ext_Acc_5_1</transition>\n"
                + "          <transition>End_Ext_Acc_2_1</transition>\n"
                + "          <transition>End_Ext_Acc_3_1</transition>\n"
                + "          <transition>End_Ext_Acc_5_2</transition>\n"
                + "          <transition>End_Ext_Acc_4_2</transition>\n"
                + "          <transition>End_Ext_Acc_3_2</transition>\n"
                + "          <transition>End_Ext_Acc_1_2</transition>\n"
                + "          <transition>End_Ext_Acc_5_3</transition>\n"
                + "          <transition>End_Ext_Acc_4_3</transition>\n"
                + "          <transition>End_Ext_Acc_2_3</transition>\n"
                + "          <transition>End_Ext_Acc_1_3</transition>\n"
                + "          <transition>End_Ext_Acc_5_4</transition>\n"
                + "          <transition>End_Ext_Acc_3_4</transition>\n"
                + "          <transition>End_Ext_Acc_2_4</transition>\n"
                + "          <transition>End_Ext_Acc_1_4</transition>\n"
                + "          <transition>End_Ext_Acc_4_5</transition>\n"
                + "          <transition>End_Ext_Acc_3_5</transition>\n"
                + "          <transition>End_Ext_Acc_2_5</transition>\n"
                + "          <transition>End_Ext_Acc_1_5</transition>\n"
                + "        </is-fireable>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-5</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <finally>\n"
                + "          <next>\n"
                + "            <is-fireable>\n"
                + "              <transition>Begin_Own_Acc_2</transition>\n"
                + "              <transition>Begin_Own_Acc_1</transition>\n"
                + "              <transition>Begin_Own_Acc_5</transition>\n"
                + "              <transition>Begin_Own_Acc_3</transition>\n"
                + "              <transition>Begin_Own_Acc_4</transition>\n"
                + "            </is-fireable>\n"
                + "          </next>\n"
                + "        </finally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-6</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <next>\n"
                + "          <until>\n"
                + "            <before>\n"
                + "              <is-fireable>\n"
                + "                <transition>Req_Ext_Acc_2</transition>\n"
                + "                <transition>Req_Ext_Acc_1</transition>\n"
                + "                <transition>Req_Ext_Acc_4</transition>\n"
                + "                <transition>Req_Ext_Acc_3</transition>\n"
                + "                <transition>Req_Ext_Acc_5</transition>\n"
                + "              </is-fireable>\n"
                + "            </before>\n"
                + "            <reach>\n"
                + "              <until>\n"
                + "                <before>\n"
                + "                  <is-fireable>\n"
                + "                    <transition>End_Own_Acc_5_5</transition>\n"
                + "                    <transition>End_Own_Acc_4_4</transition>\n"
                + "                    <transition>End_Own_Acc_1_1</transition>\n"
                + "                    <transition>End_Own_Acc_3_3</transition>\n"
                + "                    <transition>End_Own_Acc_2_2</transition>\n"
                + "                  </is-fireable>\n"
                + "                </before>\n"
                + "                <reach>\n"
                + "                  <is-fireable>\n"
                + "                    <transition>Req_Ext_Acc_2</transition>\n"
                + "                    <transition>Req_Ext_Acc_1</transition>\n"
                + "                    <transition>Req_Ext_Acc_4</transition>\n"
                + "                    <transition>Req_Ext_Acc_3</transition>\n"
                + "                    <transition>Req_Ext_Acc_5</transition>\n"
                + "                  </is-fireable>\n"
                + "                </reach>\n"
                + "              </until>\n"
                + "            </reach>\n"
                + "          </until>\n"
                + "        </next>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-7</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <globally>\n"
                + "          <globally>\n"
                + "            <globally>\n"
                + "              <is-fireable>\n"
                + "                <transition>Begin_Ext_Acc_1_2</transition>\n"
                + "                <transition>Begin_Ext_Acc_3_2</transition>\n"
                + "                <transition>Begin_Ext_Acc_4_2</transition>\n"
                + "                <transition>Begin_Ext_Acc_5_2</transition>\n"
                + "                <transition>Begin_Ext_Acc_2_1</transition>\n"
                + "                <transition>Begin_Ext_Acc_3_1</transition>\n"
                + "                <transition>Begin_Ext_Acc_4_1</transition>\n"
                + "                <transition>Begin_Ext_Acc_5_1</transition>\n"
                + "                <transition>Begin_Ext_Acc_1_4</transition>\n"
                + "                <transition>Begin_Ext_Acc_2_4</transition>\n"
                + "                <transition>Begin_Ext_Acc_3_4</transition>\n"
                + "                <transition>Begin_Ext_Acc_5_4</transition>\n"
                + "                <transition>Begin_Ext_Acc_1_3</transition>\n"
                + "                <transition>Begin_Ext_Acc_2_3</transition>\n"
                + "                <transition>Begin_Ext_Acc_4_3</transition>\n"
                + "                <transition>Begin_Ext_Acc_5_3</transition>\n"
                + "                <transition>Begin_Ext_Acc_2_5</transition>\n"
                + "                <transition>Begin_Ext_Acc_1_5</transition>\n"
                + "                <transition>Begin_Ext_Acc_4_5</transition>\n"
                + "                <transition>Begin_Ext_Acc_3_5</transition>\n"
                + "              </is-fireable>\n"
                + "            </globally>\n"
                + "          </globally>\n"
                + "        </globally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-8</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <until>\n"
                + "          <before>\n"
                + "            <next>\n"
                + "              <until>\n"
                + "                <before>\n"
                + "                  <is-fireable>\n"
                + "                    <transition>End_Ext_Acc_4_1</transition>\n"
                + "                    <transition>End_Ext_Acc_5_1</transition>\n"
                + "                    <transition>End_Ext_Acc_2_1</transition>\n"
                + "                    <transition>End_Ext_Acc_3_1</transition>\n"
                + "                    <transition>End_Ext_Acc_5_2</transition>\n"
                + "                    <transition>End_Ext_Acc_4_2</transition>\n"
                + "                    <transition>End_Ext_Acc_3_2</transition>\n"
                + "                    <transition>End_Ext_Acc_1_2</transition>\n"
                + "                    <transition>End_Ext_Acc_5_3</transition>\n"
                + "                    <transition>End_Ext_Acc_4_3</transition>\n"
                + "                    <transition>End_Ext_Acc_2_3</transition>\n"
                + "                    <transition>End_Ext_Acc_1_3</transition>\n"
                + "                    <transition>End_Ext_Acc_5_4</transition>\n"
                + "                    <transition>End_Ext_Acc_3_4</transition>\n"
                + "                    <transition>End_Ext_Acc_2_4</transition>\n"
                + "                    <transition>End_Ext_Acc_1_4</transition>\n"
                + "                    <transition>End_Ext_Acc_4_5</transition>\n"
                + "                    <transition>End_Ext_Acc_3_5</transition>\n"
                + "                    <transition>End_Ext_Acc_2_5</transition>\n"
                + "                    <transition>End_Ext_Acc_1_5</transition>\n"
                + "                  </is-fireable>\n"
                + "                </before>\n"
                + "                <reach>\n"
                + "                  <is-fireable>\n"
                + "                    <transition>Begin_Own_Acc_2</transition>\n"
                + "                    <transition>Begin_Own_Acc_1</transition>\n"
                + "                    <transition>Begin_Own_Acc_5</transition>\n"
                + "                    <transition>Begin_Own_Acc_3</transition>\n"
                + "                    <transition>Begin_Own_Acc_4</transition>\n"
                + "                  </is-fireable>\n"
                + "                </reach>\n"
                + "              </until>\n"
                + "            </next>\n"
                + "          </before>\n"
                + "          <reach>\n"
                + "            <next>\n"
                + "              <is-fireable>\n"
                + "                <transition>End_Ext_Acc_4_1</transition>\n"
                + "                <transition>End_Ext_Acc_5_1</transition>\n"
                + "                <transition>End_Ext_Acc_2_1</transition>\n"
                + "                <transition>End_Ext_Acc_3_1</transition>\n"
                + "                <transition>End_Ext_Acc_5_2</transition>\n"
                + "                <transition>End_Ext_Acc_4_2</transition>\n"
                + "                <transition>End_Ext_Acc_3_2</transition>\n"
                + "                <transition>End_Ext_Acc_1_2</transition>\n"
                + "                <transition>End_Ext_Acc_5_3</transition>\n"
                + "                <transition>End_Ext_Acc_4_3</transition>\n"
                + "                <transition>End_Ext_Acc_2_3</transition>\n"
                + "                <transition>End_Ext_Acc_1_3</transition>\n"
                + "                <transition>End_Ext_Acc_5_4</transition>\n"
                + "                <transition>End_Ext_Acc_3_4</transition>\n"
                + "                <transition>End_Ext_Acc_2_4</transition>\n"
                + "                <transition>End_Ext_Acc_1_4</transition>\n"
                + "                <transition>End_Ext_Acc_4_5</transition>\n"
                + "                <transition>End_Ext_Acc_3_5</transition>\n"
                + "                <transition>End_Ext_Acc_2_5</transition>\n"
                + "                <transition>End_Ext_Acc_1_5</transition>\n"
                + "              </is-fireable>\n"
                + "            </next>\n"
                + "          </reach>\n"
                + "        </until>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-9</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <next>\n"
                + "          <globally>\n"
                + "            <globally>\n"
                + "              <until>\n"
                + "                <before>\n"
                + "                  <is-fireable>\n"
                + "                    <transition>Begin_Own_Acc_2</transition>\n"
                + "                    <transition>Begin_Own_Acc_1</transition>\n"
                + "                    <transition>Begin_Own_Acc_5</transition>\n"
                + "                    <transition>Begin_Own_Acc_3</transition>\n"
                + "                    <transition>Begin_Own_Acc_4</transition>\n"
                + "                  </is-fireable>\n"
                + "                </before>\n"
                + "                <reach>\n"
                + "                  <is-fireable>\n"
                + "                    <transition>End_Own_Acc_5_5</transition>\n"
                + "                    <transition>End_Own_Acc_4_4</transition>\n"
                + "                    <transition>End_Own_Acc_1_1</transition>\n"
                + "                    <transition>End_Own_Acc_3_3</transition>\n"
                + "                    <transition>End_Own_Acc_2_2</transition>\n"
                + "                  </is-fireable>\n"
                + "                </reach>\n"
                + "              </until>\n"
                + "            </globally>\n"
                + "          </globally>\n"
                + "        </next>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-10</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <finally>\n"
                + "          <until>\n"
                + "            <before>\n"
                + "              <globally>\n"
                + "                <finally>\n"
                + "                  <is-fireable>\n"
                + "                    <transition>Begin_Ext_Acc_1_2</transition>\n"
                + "                    <transition>Begin_Ext_Acc_3_2</transition>\n"
                + "                    <transition>Begin_Ext_Acc_4_2</transition>\n"
                + "                    <transition>Begin_Ext_Acc_5_2</transition>\n"
                + "                    <transition>Begin_Ext_Acc_2_1</transition>\n"
                + "                    <transition>Begin_Ext_Acc_3_1</transition>\n"
                + "                    <transition>Begin_Ext_Acc_4_1</transition>\n"
                + "                    <transition>Begin_Ext_Acc_5_1</transition>\n"
                + "                    <transition>Begin_Ext_Acc_1_4</transition>\n"
                + "                    <transition>Begin_Ext_Acc_2_4</transition>\n"
                + "                    <transition>Begin_Ext_Acc_3_4</transition>\n"
                + "                    <transition>Begin_Ext_Acc_5_4</transition>\n"
                + "                    <transition>Begin_Ext_Acc_1_3</transition>\n"
                + "                    <transition>Begin_Ext_Acc_2_3</transition>\n"
                + "                    <transition>Begin_Ext_Acc_4_3</transition>\n"
                + "                    <transition>Begin_Ext_Acc_5_3</transition>\n"
                + "                    <transition>Begin_Ext_Acc_2_5</transition>\n"
                + "                    <transition>Begin_Ext_Acc_1_5</transition>\n"
                + "                    <transition>Begin_Ext_Acc_4_5</transition>\n"
                + "                    <transition>Begin_Ext_Acc_3_5</transition>\n"
                + "                  </is-fireable>\n"
                + "                </finally>\n"
                + "              </globally>\n"
                + "            </before>\n"
                + "            <reach>\n"
                + "              <is-fireable>\n"
                + "                <transition>Req_Ext_Acc_2</transition>\n"
                + "                <transition>Req_Ext_Acc_1</transition>\n"
                + "                <transition>Req_Ext_Acc_4</transition>\n"
                + "                <transition>Req_Ext_Acc_3</transition>\n"
                + "                <transition>Req_Ext_Acc_5</transition>\n"
                + "              </is-fireable>\n"
                + "            </reach>\n"
                + "          </until>\n"
                + "        </finally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-11</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <until>\n"
                + "          <before>\n"
                + "            <globally>\n"
                + "              <globally>\n"
                + "                <next>\n"
                + "                  <is-fireable>\n"
                + "                    <transition>Begin_Own_Acc_2</transition>\n"
                + "                    <transition>Begin_Own_Acc_1</transition>\n"
                + "                    <transition>Begin_Own_Acc_5</transition>\n"
                + "                    <transition>Begin_Own_Acc_3</transition>\n"
                + "                    <transition>Begin_Own_Acc_4</transition>\n"
                + "                  </is-fireable>\n"
                + "                </next>\n"
                + "              </globally>\n"
                + "            </globally>\n"
                + "          </before>\n"
                + "          <reach>\n"
                + "            <until>\n"
                + "              <before>\n"
                + "                <next>\n"
                + "                  <is-fireable>\n"
                + "                    <transition>Begin_Own_Acc_2</transition>\n"
                + "                    <transition>Begin_Own_Acc_1</transition>\n"
                + "                    <transition>Begin_Own_Acc_5</transition>\n"
                + "                    <transition>Begin_Own_Acc_3</transition>\n"
                + "                    <transition>Begin_Own_Acc_4</transition>\n"
                + "                  </is-fireable>\n"
                + "                </next>\n"
                + "              </before>\n"
                + "              <reach>\n"
                + "                <finally>\n"
                + "                  <is-fireable>\n"
                + "                    <transition>Req_Ext_Acc_2</transition>\n"
                + "                    <transition>Req_Ext_Acc_1</transition>\n"
                + "                    <transition>Req_Ext_Acc_4</transition>\n"
                + "                    <transition>Req_Ext_Acc_3</transition>\n"
                + "                    <transition>Req_Ext_Acc_5</transition>\n"
                + "                  </is-fireable>\n"
                + "                </finally>\n"
                + "              </reach>\n"
                + "            </until>\n"
                + "          </reach>\n"
                + "        </until>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-12</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <globally>\n"
                + "          <is-fireable>\n"
                + "            <transition>Begin_Ext_Acc_1_2</transition>\n"
                + "            <transition>Begin_Ext_Acc_3_2</transition>\n"
                + "            <transition>Begin_Ext_Acc_4_2</transition>\n"
                + "            <transition>Begin_Ext_Acc_5_2</transition>\n"
                + "            <transition>Begin_Ext_Acc_2_1</transition>\n"
                + "            <transition>Begin_Ext_Acc_3_1</transition>\n"
                + "            <transition>Begin_Ext_Acc_4_1</transition>\n"
                + "            <transition>Begin_Ext_Acc_5_1</transition>\n"
                + "            <transition>Begin_Ext_Acc_1_4</transition>\n"
                + "            <transition>Begin_Ext_Acc_2_4</transition>\n"
                + "            <transition>Begin_Ext_Acc_3_4</transition>\n"
                + "            <transition>Begin_Ext_Acc_5_4</transition>\n"
                + "            <transition>Begin_Ext_Acc_1_3</transition>\n"
                + "            <transition>Begin_Ext_Acc_2_3</transition>\n"
                + "            <transition>Begin_Ext_Acc_4_3</transition>\n"
                + "            <transition>Begin_Ext_Acc_5_3</transition>\n"
                + "            <transition>Begin_Ext_Acc_2_5</transition>\n"
                + "            <transition>Begin_Ext_Acc_1_5</transition>\n"
                + "            <transition>Begin_Ext_Acc_4_5</transition>\n"
                + "            <transition>Begin_Ext_Acc_3_5</transition>\n"
                + "          </is-fireable>\n"
                + "        </globally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-13</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <globally>\n"
                + "          <globally>\n"
                + "            <next>\n"
                + "              <until>\n"
                + "                <before>\n"
                + "                  <is-fireable>\n"
                + "                    <transition>Begin_Own_Acc_2</transition>\n"
                + "                    <transition>Begin_Own_Acc_1</transition>\n"
                + "                    <transition>Begin_Own_Acc_5</transition>\n"
                + "                    <transition>Begin_Own_Acc_3</transition>\n"
                + "                    <transition>Begin_Own_Acc_4</transition>\n"
                + "                  </is-fireable>\n"
                + "                </before>\n"
                + "                <reach>\n"
                + "                  <is-fireable>\n"
                + "                    <transition>End_Own_Acc_5_5</transition>\n"
                + "                    <transition>End_Own_Acc_4_4</transition>\n"
                + "                    <transition>End_Own_Acc_1_1</transition>\n"
                + "                    <transition>End_Own_Acc_3_3</transition>\n"
                + "                    <transition>End_Own_Acc_2_2</transition>\n"
                + "                  </is-fireable>\n"
                + "                </reach>\n"
                + "              </until>\n"
                + "            </next>\n"
                + "          </globally>\n"
                + "        </globally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-14</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <globally>\n"
                + "          <is-fireable>\n"
                + "            <transition>End_Ext_Acc_4_1</transition>\n"
                + "            <transition>End_Ext_Acc_5_1</transition>\n"
                + "            <transition>End_Ext_Acc_2_1</transition>\n"
                + "            <transition>End_Ext_Acc_3_1</transition>\n"
                + "            <transition>End_Ext_Acc_5_2</transition>\n"
                + "            <transition>End_Ext_Acc_4_2</transition>\n"
                + "            <transition>End_Ext_Acc_3_2</transition>\n"
                + "            <transition>End_Ext_Acc_1_2</transition>\n"
                + "            <transition>End_Ext_Acc_5_3</transition>\n"
                + "            <transition>End_Ext_Acc_4_3</transition>\n"
                + "            <transition>End_Ext_Acc_2_3</transition>\n"
                + "            <transition>End_Ext_Acc_1_3</transition>\n"
                + "            <transition>End_Ext_Acc_5_4</transition>\n"
                + "            <transition>End_Ext_Acc_3_4</transition>\n"
                + "            <transition>End_Ext_Acc_2_4</transition>\n"
                + "            <transition>End_Ext_Acc_1_4</transition>\n"
                + "            <transition>End_Ext_Acc_4_5</transition>\n"
                + "            <transition>End_Ext_Acc_3_5</transition>\n"
                + "            <transition>End_Ext_Acc_2_5</transition>\n"
                + "            <transition>End_Ext_Acc_1_5</transition>\n"
                + "          </is-fireable>\n"
                + "        </globally>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "  <property>\n"
                + "    <id>SharedMemory-COL-000005-LTLFireability-15</id>\n"
                + "    <description>Automatically generated</description>\n"
                + "    <formula>\n"
                + "      <all-paths>\n"
                + "        <is-fireable>\n"
                + "          <transition>End_Ext_Acc_4_1</transition>\n"
                + "          <transition>End_Ext_Acc_5_1</transition>\n"
                + "          <transition>End_Ext_Acc_2_1</transition>\n"
                + "          <transition>End_Ext_Acc_3_1</transition>\n"
                + "          <transition>End_Ext_Acc_5_2</transition>\n"
                + "          <transition>End_Ext_Acc_4_2</transition>\n"
                + "          <transition>End_Ext_Acc_3_2</transition>\n"
                + "          <transition>End_Ext_Acc_1_2</transition>\n"
                + "          <transition>End_Ext_Acc_5_3</transition>\n"
                + "          <transition>End_Ext_Acc_4_3</transition>\n"
                + "          <transition>End_Ext_Acc_2_3</transition>\n"
                + "          <transition>End_Ext_Acc_1_3</transition>\n"
                + "          <transition>End_Ext_Acc_5_4</transition>\n"
                + "          <transition>End_Ext_Acc_3_4</transition>\n"
                + "          <transition>End_Ext_Acc_2_4</transition>\n"
                + "          <transition>End_Ext_Acc_1_4</transition>\n"
                + "          <transition>End_Ext_Acc_4_5</transition>\n"
                + "          <transition>End_Ext_Acc_3_5</transition>\n"
                + "          <transition>End_Ext_Acc_2_5</transition>\n"
                + "          <transition>End_Ext_Acc_1_5</transition>\n"
                + "        </is-fireable>\n"
                + "      </all-paths>\n"
                + "    </formula>\n"
                + "  </property>\n"
                + "</property-set>";
        PetriNet net = new PnmlPNParser().parseFile(inputDir + "/SharedMemory-PT-000005/model.pnml");
        Map<String, ILTLFormula> forms = MCCXMLFormulaParser.parseLTL(formulas, net);
//        System.out.println(forms.toString());
    }
}
